#ifndef CoCoA_sugar_H
#define CoCoA_sugar_H

//   Copyright (c)  2009  Anna M. Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/SmartPtrIRC.H"

#include <iosfwd>
//using std::ostream;
#include <cstddef>
//using std::size_t;

namespace CoCoA
{

  class ConstRefPPMonoidElem; // forward declaration -- defined in PPMonoid.H
  class GPoly;             // forward declaration -- defined in TmpGPoly.H
  class ReductionCog;      // forward declaration -- defined in ReductionCog.H
  class degree;            // forward declaration -- defined in degree.H

  class SugarBase; // forward declaration for SmartPtrIRC
  class sweetener
  {
  public:
    explicit sweetener(SugarBase* ptr): mySmartPtr(ptr) {}
    // assignment disabled because SmartPtrIRC has no assignment
    SugarBase* operator->() { return mySmartPtr.operator->(); }  ///< Allow member fns to be called.
    const SugarBase* operator->() const { return mySmartPtr.operator->(); }  ///< Allow const member fns to be called.
    bool operator==(const sweetener& f) const { return mySmartPtr==f.mySmartPtr; }
  private: // data members
    SmartPtrIRC<SugarBase> mySmartPtr;
  };

  std::ostream& operator<<(std::ostream& out, const sweetener& s);

  class SugarBase: protected IntrusiveReferenceCount
  {
    friend class SmartPtrIRC<SugarBase>; // morally "friend sweetener", so it can alter reference count
  protected:
    SugarBase(): IntrusiveReferenceCount() {}
  public:
    virtual ~SugarBase() {};
    ///@name Warning! Throw an error if the wrong type of value is asked!
    //@{
    virtual const degree& myWSugar() =0; ///< only if impl stores this value
    virtual long myStdSugar() =0; ///< only if impl stores this value
    //@}
    virtual void myUpdate(ReductionCog F, GPoly& g) =0;
    virtual void myUpdate(ConstRefPPMonoidElem CofactorPP, GPoly& g) =0;
    virtual std::ostream& myOutput(std::ostream& out) const =0;
  };

  ///@name pseudo-constructors
  //@{
  sweetener NewStdSugar(long s);
  sweetener NewWSugar(const degree& s);
  sweetener NewSatStdSugar(long s);
  sweetener NewSatWSugar(const degree& s);
  //@}

  //------- inline functions ----------------------------

  // none

} // end of namespace CoCoA


//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/sugar.H,v 1.1 2009/02/09 13:57:05 bigatti Exp $
// $Log: sugar.H,v $
// Revision 1.1  2009/02/09 13:57:05  bigatti
// -- first import
//

#endif
