//   Copyright (c)  2009  Anna M. Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/PPMonoid.H"
#include "CoCoA/ReductionCog.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/TmpGPoly.H"
#include "CoCoA/assert.H"
#include "CoCoA/sugar.H"

#include <algorithm>
using std::max; // for std sugar
#include <iostream>
//using std::ostream;
#include <cstddef>
//using std::size_t;
#include <vector>
//using std::vector;
#include <memory>
//using std::auto_ptr;


namespace CoCoA
{

  std::ostream& operator<<(std::ostream& out, const sweetener& s)
  { return s->myOutput(out); }


  namespace SugarTypes
  {

    class StdDegPlain: public SugarBase
    {
    public:
      StdDegPlain(long s): SugarBase() { myValue = s; }
      //??      ~StdDegPlain()  {};
      virtual const degree& myWSugar() { CoCoA_ERROR("Wrong type for this sugar", "SugarTypes::StdDegPlain"); static degree NeverUsed(0); return NeverUsed; }
      virtual long myStdSugar() { return myValue; }
      virtual void myUpdate(ReductionCog F, GPoly& g) { myUpdate(ActiveLPP(F)/LPPForOrd(g), g); }
      virtual void myUpdate(ConstRefPPMonoidElem CofactorPP, GPoly& g);
      virtual std::ostream& myOutput(std::ostream& out) const { return out << myValue; }
    private:
      long myValue;
    };


    class WDegPlain: public SugarBase
    {
    public:
      WDegPlain(const degree& s): SugarBase(), myValue(s) {}
      //??      ~WDegPlain()  {};
      virtual const degree& myWSugar() { return myValue; }
      virtual long myStdSugar() { CoCoA_ERROR("Wrong type for this sugar", "SugarTypes::WDegPlain"); return 0;}
      virtual void myUpdate(ReductionCog F, GPoly& g) { myUpdate(ActiveLPP(F)/LPPForOrd(g), g); }
      virtual void myUpdate(ConstRefPPMonoidElem CofactorPP, GPoly& g);
      virtual std::ostream& myOutput(std::ostream& out) const { return out << myValue; }
    private:
      degree myValue;
    };



  }  // namespace sugar




//--  StdDegPlain  ----------------------------------

  void SugarTypes::StdDegPlain::myUpdate(ConstRefPPMonoidElem CofactorPP, GPoly& g)
  {
    myValue = std::max(myValue, g.myGRingInfo().TmpStdDeg(CofactorPP) + sugar(g));
  }

//--  WDegPlain  ----------------------------------

  void SugarTypes::WDegPlain::myUpdate(ConstRefPPMonoidElem CofactorPP, GPoly& g)
  {
    myValue = top(myValue, wdeg(CofactorPP) + wsugar(g)); /// never tested
  }



  //----------------------------------------------------------------------
  // Here are the pseudo-constructors:

  sweetener NewStdSugar(long s)
  { return sweetener(new SugarTypes::StdDegPlain(s)); }

  sweetener NewWSugar(const degree& s)
  { return sweetener(new SugarTypes::WDegPlain(s)); }

//   sweetener NewSatStdSugar(long s)
//   { return sweetener(new SugarTypes::StdSatAlg(s)); }

//   sweetener NewSatWSugar(const degree& s)
//   { return sweetener(new SugarTypes::WSatAlg(s)); }


} // namespace CoCoA



//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/sugar.C,v 1.1 2009/02/09 13:57:05 bigatti Exp $
// $Log: sugar.C,v $
// Revision 1.1  2009/02/09 13:57:05  bigatti
// -- first import
//
